#define __KERNEL__

/* Saint Michael, Linux Kernel Module.
 * Verions: 0.03
 *
 * June 1, 2001
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/sys.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/segment.h>
#include <asm/unistd.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <asm/ptrace.h>
#include <asm/pgtable.h>
#include "StMichael_lkm.h"
#include "StMichael_string_util.h"

mm_segment_t old_fs;

void* sm_memset(void* mem,unsigned char value,unsigned int s)
{
    register unsigned int i;
    
    if(!mem || !s) return NULL;
    for(i = 0;i < s;i++) ((unsigned char*)mem)[i] = value;
    return mem;
}

void* sm_memcpy(void* dst,const void* src,unsigned int s)
{
    register unsigned int i;
    
    if(!src || !dst || !s) return NULL;
    for(i = 0;i < s;i++) ((unsigned char*)dst)[i] = ((unsigned char*)src)[i];
    return dst;
}

int sm_memcmp(const void* s1,const void* s2,unsigned int s)
{
    register unsigned int i;
    
    if(!s1 || !s2 || !s) return 1;
    for(i = 0;i < s;i++) {
	if(((unsigned char*)s1)[i] != ((unsigned char*)s2)[i]) return 1;
    }
    return 0;
}

char* sm_strncpy(char* dst,const char* src,unsigned int s)
{
    register unsigned int i;
    
    if(!dst || !src || !s) return NULL;
    sm_memset(dst,0,s);
    for(i = 0;i < s;i++) {
	if(src[i] == '\0') break;
	dst[i] = src[i];
    }
    return dst;
}

inline int sm_strnlen(const char* s1,int max_len)
{
    register int i;

    for (i = 0; i < max_len && s1[i]; i++);
    return i;
}

int sm_strlen(const char* s1)
{
    register unsigned int i;

    if(!s1) return 0;
    for(i = 0;s1[i];i++);
    return i;
}

inline int sm_strncmp(const char* s1,const char* s2,int max_len)
{
    register int i;

    for (i = 0; i < max_len && ( s1[i] != '\0' && s2[i] != '\0') ; i++)
	if (s1[i] != s2[i])
	    return 1;
    return 0;
}

int sm_strcmp(const char* s1,const char* s2)
{
    register unsigned int i;

    if(!s1 || !s2) return 1;
    for(i = 0;(s1[i] != '\0' && s2[i] != '\0');i++) {
	if(s1[i] != s2[i]) return 1;
    }
    return 0;
}

char* sm_substr(char* s,int start,int end)
{
    int i;
    char* buf = NULL;

    if(!(buf = kmalloc(sizeof(char) * (end - start + 1),GFP_KERNEL)))
	return NULL;
    for (i = 0; i < (end - start + 1); i++) buf[i] = '\0';
    for (i = start; i <= end; i++) buf[i] = s[i];
    return buf;
}

int count(char **argv, int max)
{
    int             i = 0;

    if (argv != NULL) {
	for (;;) {
	    char           *p;
	    int             error;

	    error = get_user(p, argv);
	    if (error)
		return error;
	    if (!p)
		break;
	    argv++;
	    if (++i > max)
		return -E2BIG;
	}
    }
    return i;
}

char* sm_strdup(const char* str)
{
    char* ret = NULL;
    unsigned int s = 0;
    
    if(!str) return NULL;
    s = sm_strlen(str) + 1;
    if(!(ret = (char*) kmalloc(s,GFP_KERNEL))) return NULL;
    sm_memset(ret,0,s);
    sm_strncpy(ret,str,s - 1);
    return ret;
}

/*  This is a more secure (I hope...) replacement for printk. It's a function
    local to this module and its address isn't exported to the kernel, so it
    isn't so easy to modify as printk. Unfortunally it uses sys_write() and
    isn't able to log anything... */

int sm_printf(const char* fmt,...)
{
    va_list args;
    char buf[4096];
    static int (*write)(unsigned int,char*,ssize_t);
    static unsigned char init = 0;
	
    if(!fmt) return -1;
    if(!init) {
	write = sys_call_table[SYS_write];
	init = 1;
    }
    sm_memset(buf,0,sizeof(buf));
    va_start(args,fmt);
    vsprintf(buf,fmt,args);
    va_end(args);
    BEGIN_KMEM
    write(1,buf,sm_strlen(buf));
    END_KMEM
    return 0;
}
